local L = AceLibrary("AceLocale-2.2"):new("NinjutFu")
local crayon = AceLibrary("Crayon-2.0")

NinjutFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceEvent-2.0")
NinjutFu.hasIcon = "Interface\\Icons\\Ability_Vanish"

NinjutFu:RegisterDB("NinjutFuDB")
NinjutFu:RegisterDefaults("profile", {
	threshCount = 20,
	blindCount = 20,
	teaCount = 10,
	buyFlash = true,
	showFlash = true,
	showBlind = true,
	showTea = true,
	compactMode = false,
	showTipTitles = true,
	tooltipSkipCategories = "",
	highend = false,
	items = "all",
})

-- A long string with itemids of all items supported sorted into categories in the format
-- ";Category1:1234:1235;Category2:2345:2346:2347;"
local items = ";Reagents:5140:5530;" -- Flash and Blinding Powder
	.. "Energy:7676:27553;" -- Thistle Tea and Crimson Steer
	.. "Poisons"
	.. ":21835" -- Anastethic
	.. ":3775:3776" -- Crippling
	.. ":2892:2893:8984:8985:20844:22053:22054" -- Deadly Poison
	.. ":6947:6949:6950:8926:8927:8928:21927" -- Instant Poison
	.. ":5237:6951:9186" -- Mind-numbing
	.. ":10918:10920:10921:10922:22055" -- Wound Poison
	.. ";"
	.. "Stones"
	-- Rough Sharpening Stone, Rough Weightstone, Coarse Sharpening Stone, Coarse Weightstone, Heavy Sharpening Stone, Heavy Weightstone, 
	-- Solid Sharpening Stone, Solid Weightstone, Dense Sharpening Stone, Dense Weightstone, Consecrated Sharpening Stone, 
	-- Elemental Sharpening Stone, Fel Sharpening Stone, Fel Weightstone, Adamantite Sharpening Stone, Adamantite Weightstone
	-- in that order...
	.. ":2862:3239:2863:3240:2871:3241:7964:7965:12404:12643:23122:18262:23528:28420:23529:28421;"

-- A few functions using the string items above

-- Returns the category for a given itemid
-- (not used anywhere atm, just wrote it while I had luas magic patterns fresh in the head :P)
function NinjutFu:GetCategoryForItem(itemid)
	local found, _, category = string.find(items, ";([^:]+)[:%d]+:"..itemid.."[:;]")
	return found ~= nil and category or nil
end

-- To iterate all categories of items:
-- for category in NinjutFu:IterItemCategories() do
function NinjutFu:IterItemCategories()
	local pos = 0
	return function()
		local category
		pos, _, category = string.find(items, ";([^:]+):", pos+1)
		return pos ~= nil and category or nil
	end
end

-- To iterate all itemids in items:
-- for itemid in NinjutFu:IterAllItems() do
function NinjutFu:IterAllItems()
	local pos = 0
	return function()
		local item
		pos, _, item = string.find(items, ":(%d+)[:;]", pos+1)
		return pos ~= nil and tonumber(item) or nil
	end
end

-- To iterate all itemids in for example the category "Poisons":
-- for itemid in NinjutFu:IterCategoryItems(L["Poisons"]) do
function NinjutFu:IterCategoryItems(category)
	local pos = 0
	local _, _, items = string.find(items, ";"..category.."(:[%d:]+;)")
	return function()
		local item
		pos, _, item = string.find(items, ":(%d+)[:;]", pos+1)
		return pos ~= nil and tonumber(item) or nil
	end
end

function NinjutFu:PopulateItemCounts()
	local set = self.db.profile.items

	self.itemCounts = {}
	if set == "all" then
		for itemid in self:IterAllItems() do
			self.itemCounts[itemid] = 0
		end
		return true
	elseif set == "none" then
		return true
	elseif set == "he60" then
		set = "5140 5530 7676 3775 3776 20844 8928 9186 10922 12404 12643 23122 18262"
	elseif set == "he70" then
		set = "5140 5530 7676 27553 21835 3775 3776 22054 21927 9186 22055 23528 28420 23529 28421"
	end

	local itemid
	local pos = 0
	while true do
		_, pos, itemid = string.find(set, "^ *([^%s]+) *", pos + 1);
		if not itemid then
			break
		end
		self.itemCounts[tonumber(itemid)] = 0
	end
end

function NinjutFu:OnInitialize()
end

function NinjutFu:OnEnable()
	self:PopulateItemCounts()

	self:RegisterBucketEvent("BAG_UPDATE", 0.5, "Update")
	self:RegisterEvent("MERCHANT_SHOW", "RefillFlash")
end

function NinjutFu:OnDisable()
	self.itemCounts = nil
end

function NinjutFu:OnDataUpdate()
	self:CountItems()
end

function NinjutFu:OnTextUpdate()
	local barText = self:GetTitle()

	local notFirst = false
	local seperator = self.db.profile.compactMode and "/" or " "

	if (self.db.profile.showFlash) then
		barText = self:Textify(self.itemCounts[5140], "F", self.db.profile.threshCount)
		notFirst = true
	end
	if (self.db.profile.showBlind) then
		if (notFirst) then
			barText = barText .. seperator .. self:Textify(self.itemCounts[5530], "B", 20)
		else
			barText = self:Textify(self.itemCounts[5530], "B", 20)
			notFirst = true
		end
	end
	if (self.db.profile.showTea) then
		if (notFirst) then
			barText = barText .. seperator .. self:Textify(self.itemCounts[7676], "T", 10)
		else
			barText = self:Textify(self.itemCounts[7676], "T", 10)
		end
	end

	self:SetText(barText)
end

function NinjutFu:Textify(count, powder, stacksize)
	if (self.db.profile.compactMode) then
		return self:Colorize(count, stacksize)
	else
		return self:Colorize(count, stacksize) .. powder
	end
end

function NinjutFu:Colorize(count, stacksize)
	return crayon:Colorize(crayon:GetThresholdHexColor(count, stacksize/4, stacksize/2, stacksize*0.65, stacksize*0.8, stacksize*0.95), count)
end

function NinjutFu:CountItems()
	local itemid, count

	for itemid in pairs(self.itemCounts) do
		self.itemCounts[itemid] = 0
	end


	for i = NUM_BAG_FRAMES, 0, -1 do
		for j = GetContainerNumSlots(i), 1, -1 do
			itemid = self:GetItemID(i, j)
			if (self.itemCounts[itemid]) then
				_, count = GetContainerItemInfo(i, j)
				self.itemCounts[itemid] = self.itemCounts[itemid] + count
			end
		end
	end
end

function NinjutFu:GetItemID(bag, slot)
	local linktext = GetContainerItemLink(bag, slot)
	if linktext then
		local _,_,itemid = string.find(linktext, "item:(%d+):")
		return tonumber(itemid)
	else
		return nil
	end
end

function NinjutFu:UseByItemID(itemid)
	local item
	for i=0, NUM_BAG_FRAMES do
		for j=1, GetContainerNumSlots(i) do
			item = self:GetItemID(i,j)
			if item == itemid then
				UseContainerItem(i,j)
			end
		end
	end
end

-- Broken :'(
function NinjutFu:ApplyPoison(poison, slot)
	self:UseByItemID(poison)
	PickupInventoryItem(slot)
	EquipCursorItem(slot)
end


function NinjutFu:RefillFlash()
	local _, class = UnitClass("player")
	if (class == "ROGUE" and self.db.profile.buyFlash) then
		local numItems = GetMerchantNumItems()
		local currentCount = self.itemCounts[5140]
		local diff = self.db.profile.threshCount - currentCount
		if (diff > 0) then
			local link, itemid
			for i=0, numItems do
				link = GetMerchantItemLink(i)
				if (link) then
					_,_,itemid = string.find(link, "item:(%d+):")
					if tonumber(itemid) == 5140 then
						while (diff > 20) do
							BuyMerchantItem(i, 20)
							diff = diff - 20
						end
						BuyMerchantItem(i, diff)
					end
				end
			end
		end
	end
end
